/*
 * Engineering Software Package GPIB-1014P functions
 * Rev B
 * (c) Copyright 1990, National Instruments
 * All rights reserved.
 ****************************************************************
 * Note! This program has been modified from the original to work
 * on a SPARC based VME crate at Princeton University.  Modifications 
 * from original indicated by '****PUEjP'
 *
 * 5/4/98 E.Prebys  Modified from supplied source code.  Brief summary 
 *                  of changes: 
 *                  (1) Memory mapping a bit more complex under SunOS.
 *                      Must open unit and map to a page. Affected routines:
 *                             In() - no longer mapped to input()
 *                             Out() - no longer mapped to output()
 *                             ibonl() - now calls esp488_open()
 *                  (2) Changed all (char) defs to (unsigned char) to prevent
 *                      left fill problems.
 ****************************************************************
 * The GPIB functions implemented here are a straightforward
 * set of independent functions that can be used to directly
 * manipulate an interface board.  They are at the level of
 * example code provided in the hardware reference manual but
 * structured to be a subset of the complete integrated driver
 * package.  Sufficient functionality is implemented so that
 * an application program can control and coordinate data
 * transfers among a suite of instruments on the GPIB.  The
 * primary purpose of these functions is to demonstrate the
 * use of the GPIB interface board in an ATE environment so
 * that an engineer or programmer is able to customize the
 * GPIB functions for a dedicated stand-alone or ROM-based
 * system, rather than implement the software from scratch.
 * Whenever an operating system environment is available the
 * recommended approach is to use a specific binary driver or
 * handler package containing the full GPIB capability rather
 * than use this engineering software package.
 *
 * The function calls support synchronous, non-interrupt,
 * non-DMA I/O, for a single interface board which is always
 * the system controller and the controller in charge.  All
 * functions return the subset of the standard GPIB status
 * bit vector consisting of END,SRQI,CIC,ATN,LACS and TACS status bits.
 * The result of the last call is also available in the global
 * variable, ibsta.  If it was an I/O operation the actual
 * count transferred is available in the global variable,
 * ibcnt.  Before any other call is made the ibonl function
 * must be called to initialize the GPIB interface.  Prior to
 * calling ibrd or ibwrt the appropriate devices, including
 * the interface board, must be addressed by calling ibcmd
 * with the proper addressing commands.
 */
/*****PUEjP Need some extra stuff for the VME interface Under SunOS */
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sbusdev/vme.h>
#include <sys/mman.h>
#include <sys/file.h>

/* Some constants */
#ifndef ESP488_BASE
#define ESP488_BASE 0x1000      /* Base address */
#endif
#define ESP488_WINDOW_SIZE 0x10 /* window size */

typedef unsigned char byte;  /* define a byte type */
struct ibregs *ib;     /* pointer to the base address of the module */

/* New definition of In and Out */
#define In(XX) (ib->XX)
#define Out(XX,DD) ib->XX = (unsigned char) DD
/******PUEjP End of addition */

/******PUEjP This definition is no longer necessary *****/
/* #define IB      ((struct ibregs *) 0x1000)    *//* short I/O address of */
		                                 /* interface board      */

unsigned ibsta;                 /* status bits returned by last call */
unsigned ibcnt;                 /* actual byte count of last I/O transfer */

unsigned long int timeout = 0xFFFFF;   /* number of loops to count before timing out */

/****** PUEjP changed this def from char to byte (unsigned char) */
byte stat1, stat2;         /* software copies of isr1 and isr2; These bits */
                           /* are cleared when read, so each function must */
                           /* update a software copy of them for use in */
                           /* updating ibsta.  If the updated() function */
                           /* actually read these registers, it would */
                           /* clear some bits which might be needed by the */
                           /* next function to be executed. */


/* Hardware register definitions */
/*******PUEjP These have all been changed to unsigned char (byte) */
struct ibregs {            /*  write-only hardware registers       */
	byte    x0,     cdor;   /*  +1 byte out register                */
	byte    x2,     imr1;   /*  +3 interrupt mask register 1        */
	byte    x4,     imr2;   /*  +5 interrupt mask register 2        */
	byte    x6,     spmr;   /*  +7 serial poll mode register        */
	byte    x8,     admr;   /*  +9 address mode register            */
	byte    xA,     auxmr;  /*  +B auxiliary mode register          */
	byte    xC,     adr;    /*  +D address register 0/1             */
	byte    xE,     eosr;   /*  +F end of string register           */
};

/* Read-only register mnemonics corresponding to write-only registers */

#define dir     cdor
#define isr1    imr1
#define isr2    imr2
#define spsr    spmr
#define adsr    admr
#define cptr    auxmr
#define adr0    adr
#define adr1    eosr

/* Control masks for hidden registers (auxmr) */

#define ICR     0040
#define PPR     0140
#define AUXRA   0200
#define AUXRB   0240
#define AUXRE   0300

/* Hardware register bit definitions */
/*      Name            Bit(s)          register       */

#define HR_DI           (1<<0)          /* isr1         */
#define HR_DO           (1<<1)          /* isr1         */
#define HR_END          (1<<4)          /* isr1         */
#define HR_CO           (1<<3)          /* isr2         */
#define HR_SRQI         (1<<6)          /* isr2         */
#define HR_DMAI         (1<<4)          /* imr2         */
#define HR_DMAO         (1<<5)          /* imr2         */
#define HR_ADM0         (1<<0)          /* admr         */
#define HR_TRM0         (1<<4)          /* admr         */
#define HR_TRM1         (1<<5)          /* admr         */
#define HR_DL           (1<<5)          /* adr          */
#define HR_DT           (1<<6)          /* adr          */
#define HR_ARS          (1<<7)          /* adr          */
#define HR_PPU          (1<<4)          /* ppr          */

/* 7210 Auxiliary Commands */

#define AUX_PON         000     /* Immediate Execute pon        */
#define AUX_CR          002     /* Chip Reset                   */
#define AUX_SEOI        006     /* Send EOI                     */
#define AUX_TCA         021     /* Take Control Asynchronously  */
#define AUX_GTS         020     /* Go To Standby                */
#define AUX_EPP         035     /* Execute Parallel Poll        */
#define AUX_SIFC        036     /* Set IFC                      */
#define AUX_CIFC        026     /* Clear IFC                    */
#define AUX_SREN        037     /* Set REN                      */
#define AUX_CREN        027     /* Clear REN                    */


/* I/O macros; The In and Out functions have been defined as these two */
/* macros which pass the I/O address and data width to two functions */
/* called input and output.  This allows you to define your own method */
/* of accessing the registers, since some special steps may need to be */
/* taken if the GPIB-1014 is in another chassis linked by an extender */
/* or some other unusual setup. */

/*****PUEjP These are no longer valid (New defs above) 
#define In(x)           input((long)(&(IB->x)), 1)
#define Out(x,a)        output((long)(&(IB->x)), (long)a, 1)
******PUEjP End of change */

/****PUEjP These next routines are obsolete.  Prepend OBSOLETE to name */
OBSOLETE_input(address, width)               /* reads a register on the GPIB-1014; */
   long address;                    /* This input function is for a cpu */
   int width;                       /* with the same addressing and byte */
                                    /* ordering scheme as the VMEbus. If */
   {                                /* you have an Intel processor or a */
   switch(width)                    /* processor which is incapable of */
      {                             /* 32-bit accesses, you may need to */
      case 1:                       /* write your own input routine. */
         return *(char *)address;
      case 2:
         return *(int *)address;
      case 4:
         return *(long *)address;
      }
   }


OBSOLETE_output(address, value, width)       /* writes a register on the GPIB-1014 */
   long address, value;             /* This output function is for a cpu */
   int width;                       /* with the same addressing and byte */
                                    /* ordering scheme as the VMEbus. If */
   {                                /* you have an Intel processor or a */
   switch(width)                    /* processor which is incapable of */
      {                             /* 32-bit accesses, you may need to */
      case 1:                       /* write your own output routine. */
         *(char *)address = value;
         break;
      case 2:
         *(int *)address = value;
         break;
      case 4:
         *(long *)address = value;
         break;
      }
   }

/*******PUEjP End of change **/


unsigned int updated(status)     /* updates the global variable ibsta */
   unsigned int status;

/* the meanings of the bits in ibsta are:
   
   Bit   Mnemonic    Condition flagged

   13    ERR         The operation was unable to complete successfully
   12    ---
   11    END         A read operation was halted by EOI
   10    SRQI        SRQ was asserted at least once since the last operation
   9     ---
   8     ---
   7     ---
   6     ---
   5     CIC         The GPIB-1014 is controller in charge
   4     ATN         ATN is asserted
   3     TACS        The GPIB-1014 is addressed to talk
   2     LACS        The GPIB-1014 is addressed to listen
   1     ---
   0     ---

*/

   {
   ibsta &= 0x8000;                             /* clear all but ERR bit */
   ibsta |= ( (stat1 & 0x04) ? 0x8000:0x00 );   /* ERR */
   ibsta |= ( (In(adsr) & 0x02) ? 0x08:0x00 );  /* TA */
   ibsta |= ( (In(adsr) & 0x04) ? 0x04:0x00 );  /* LA */
   ibsta |= ( (In(adsr) & 0x40) ? 0x00:0x10 );  /* ATN */
   ibsta |= ( (In(adsr) & 0x80) ? 0x20:0x00 );  /* CIC */
   ibsta |= ( (stat2 & 0x40) ? 0x1000:0x00 );   /* SRQI */
   ibsta |= ( (stat1 & 0x10) ? 0x2000:0x00 );   /* END */
   return ibsta;
   }


/*
 * IBRD
 * Read up to cnt bytes of data from the GPIB into buf.  In
 * addition to I/O complete, the read operation terminates
 * on detection of EOI.  Prior to beginning the read the
 * interface is placed in the controller standby state.  No
 * handshake holdoffs are used so care must be exercised
 * when taking control (i.e., asserting ATN) following ibrd.
 * Prior to calling ibrd, the intended devices as well as
 * the interface board itself must be addressed by calling
 * ibcmd.
 */
ibrd(buf,cnt) byte buf[];{
   unsigned long int i;                      /* time counter */

	Out(auxmr, AUX_GTS);                      /* if CAC, go to standby */
	ibcnt = 0;                                /* reset count */
   ibsta = 0;                                /* clear ibsta */
	stat1 = 0;                                /* clear our isr1 copy */
   i = timeout;
   /*****PUEjP pulled out this print statement for silent operation */
   /* printf("\n  Waiting for %d bytes...\n", cnt); */
	while((ibcnt<cnt) && !(stat1 & HR_END) && i)    /* get bytes until count filled, 
                                                      EOI received, or timeout occurs */
      {
		while(i && !(stat1 = In(isr1)))        /* wait for Data In or timeout */
         i--;                                /* decrement time counter */
      if (i)
		   buf[ibcnt++]= In(dir);                 /* store byte and increment count */
      else                                   /* handle timeout */
         {
         printf("\n  Error - timed out.\n");
         ibsta |= 0x8000;
         }
      }
   stat2 = In(isr2);                /* update isr2 copy */
	return updated(ibsta); }

/*
 * IBWRT
 * Write cnt bytes of data from buf to the GPIB.  The write
 * operation terminates only on I/O complete.  By default,
 * EOI is always sent along with the last byte.  Prior to
 * beginning the write the interface is placed in the
 * controller standby state.  Prior to calling ibwrt, the
 * intended devices as well as the interface board itself
 * must be addressed by calling ibcmd.
 */
ibwrt(buf,cnt) byte buf[];{
   unsigned long int i;               /* timeout counter */

	Out(auxmr, AUX_GTS);               /* if CAC, go to standby */
	ibcnt= 0;                          /* reset count */
   ibsta = 0;                         /* clear ibsta */
   i = timeout;
	while(i && ibcnt<cnt){
		if(ibcnt==cnt-1)                /* send EOI with last byte */
			Out(auxmr, AUX_SEOI);
		Out(cdor, buf[ibcnt++]);                     /* output byte and increment count */
		while(i && !((stat1 = In(isr1)) & HR_DO))    /* wait for Data Out */
         --i;
		}
   if (!i)
      {
      printf("\n  Error - timed out.\n");
      ibsta |= 0x8000;
      }
   stat2 = In(isr2);                   /* update isr2 copy */
	return updated(ibsta); }

/*
 * IBCMD
 * Write cnt command bytes from buf to the GPIB.  The
 * command operation terminates only on I/O complete.  Prior
 * to beginning the command the interface is placed in the
 * controller active state.  Before calling ibcmd for the
 * first time, ibsic must be called to initialize the GPIB
 * and enable the interface to leave the controller idle
 * state.
 */
ibcmd(buf,cnt) byte buf[];{

	Out(auxmr, AUX_TCA);                       /* if standby, go to CAC */
	ibcnt = 0;                                 /* clear ibcnt */
   ibsta = 0;                                 /* clear ibsta */
	stat2 = 0;
	while(ibcnt<cnt){
		stat2 &= ~HR_CO;                          /* clear saved copy of Command Out */
		Out(cdor, buf[ibcnt++]);                  /* output command and increment count */
		while(((stat2 |= In(isr2)) & HR_CO)==0) ; /* wait for Command Out; "or" to preserve SRQI */
		}
   stat1 = In(isr1);                            /* update isr1 copy */
	return updated(ibsta); }

/*
 * IBWAIT
 * Check or wait for a GPIB event to occur.  The mask argument
 * is a bit vector corresponding to the status bit vector.  It
 * has a bit set for each condition which can terminate the wait
 * (only SRQI in this implementation).  If the mask is 0 then no
 * condition is waited for and the current status is simply
 * returned.  Note that since the hardware SRQI bit is volatile
 * it will only be reported once for each occurrence of SRQ.  If
 * another function has returned the SRQI status bit, then a
 * call to ibwait with a mask containing SRQI will never return.
 */
ibwait(mask){
   ibcnt = 0;                                /* no count for this function */
   ibsta = 0;                                /* clear ibsta */
   do
      {
      stat1 = In(isr1);                      /* update isr1 copy */
      stat2 = In(isr2);                      /* update isr2 copy */
      }
	while(!(updated(ibsta) & mask) && mask);  /* quit if pattern matched or mask = 0 */
	return ibsta; }


/*
 * IBRPP
 * Conduct a parallel poll and return the byte in buf.  Prior
 * to conducting the poll the interface is placed in the
 * controller active state.
 */
ibrpp(buf) byte *buf;{
   ibcnt = 0;                             /* no count for this function */
   ibsta = 0;                             /* clear ibsta */
	Out(auxmr, AUX_TCA);                   /* if standby, go to CAC */
	Out(auxmr, AUX_EPP);                   /* enable parallel poll */
   while (!((stat2 = In(isr2)) & HR_CO)); /* wait for the CO bit to be set */
	*buf= In(cptr);                        /* store the response byte */
   stat1 = In(isr1);                      /* update isr1 copy */
	return updated(ibsta); }

/*
 * IBONL
 * Initialize the interface hardware.  If v is non-zero then
 * the GPIB chip is enabled online.  If v is zero then it is
 * left disabled and offline.  Ibonl must be called before
 * any other function.
 */
int ibonl(v){
  /*******PUEjP Added command to open interface to module */
  static int fcall=1;
  if(fcall) {
    if(esp488_open()!=0) return;
    fcall=0;
    }
   ibcnt = 0;              /* no count for this function */
   ibsta = 0;              /* clear ibsta */
	Out(auxmr, AUX_CR);     /* reset chip */
	stat1 = In(isr1) && 0;  /* clear status registers by reading */
	stat2 = In(isr2) && 0;
	Out(imr1, 0);           /* disable all interrupts */
	Out(imr2, 0);
	Out(spmr, 0);                           /* reset serial poll response */
	Out(adr, 0);                            /* set GPIB address; MTA= 00, MLA= 00 */
	Out(adr, HR_ARS | HR_DT | HR_DL);       /* disable secondary addressing */
	Out(admr, HR_TRM1 | HR_TRM0 | HR_ADM0); /* enable dual primary addressing mode */
	Out(eosr, 0);
	Out(auxmr, AUX_CIFC);           /* by default enable system controller */
	Out(auxmr, ICR | 8);            /* set internal counter register N= 8 */
	Out(auxmr, PPR | HR_PPU);       /* parallel poll unconfigure */
	Out(auxmr, AUXRA | 0);          /* clear auxmr hidden registers */
	Out(auxmr, AUXRB | 0);
	Out(auxmr, AUXRE | 0);
	if(v) Out(auxmr, AUX_PON);      /* release pon state to bring online */
	return updated(ibsta); }


/*
 * IBSIC
 * Send IFC for at least 100 microseconds.  Ibsic must be
 * called prior to the first call to ibcmd in order to
 * initialize the bus and enable the interface to leave
 * the controller idle state.
 */
ibsic(){
	int i;

   ibcnt = 0;                 /* no count for this function */
   ibsta = 0;                 /* clear ibsta */
	Out(auxmr, AUX_SIFC);      /* assert IFC */
	/*****PUEjP This must have been a slow computer. Switch with a 
         usleep()  */ 
	/* for(i= 0; i<100; i++) ;  */  /* busy wait >=100us */
        usleep(100);               /* wait 100 usec */
	Out(auxmr, AUX_CIFC);      /* clear IFC */
   stat1 = In(isr1);          /* update isr1 copy */
   stat2 = In(isr2);          /* update isr2 copy */
	return updated(ibsta); }

/*
 * IBSRE
 * Send REN true if v is non-zero or false if v is zero.
 */
ibsre(v){
   ibcnt = 0;                              /* no count for this function */
   ibsta = 0;                              /* clear ibsta */
	Out(auxmr, (v? AUX_SREN : AUX_CREN));   /* set or clear REN */
   stat1 = In(isr1);                       /* update isr1 copy */
   stat2 = In(isr2);                       /* update isr2 copy */
	return updated(ibsta); }

/*
 * IBGTS
 * Go to the controller standby state from the controller
 * active state, i.e., turn ATN off.
 */
ibgts(){
   ibcnt = 0;                 /* no count for this function */
   ibsta = 0;                 /* clear ibsta */
	Out(auxmr, AUX_GTS);       /* go to standby */
   stat1 = In(isr1);          /* update isr1 copy */
   stat2 = In(isr2);          /* update isr2 copy */
	return updated(ibsta); }

/*
 * IBCAC
 * Return to the controller active state from the
 * controller standby state, i.e., turn ATN on.  Note
 * that in order to enter the controller active state
 * from the controller idle state, ibsic must be called.
 */
ibcac(){
   ibcnt = 0;                 /* no count for this function */
   ibsta = 0;                 /* clear ibsta */
	Out(auxmr, AUX_TCA);       /* assert ATN (asynchronously) */
   stat1 = In(isr1);          /* update isr1 copy */
   stat2 = In(isr2);          /* update isr2 copy */
	return updated(ibsta); }


/*
 * IBPAD
 * change the GPIB address of the interface board.
 * The address must be 0 through 31. ibonl resets the address to 0.
 */
ibpad(v)
   int v;

   {
   ibcnt = 0;                                         /* no count for this function */
   ibsta = 0;                                         /* clear ibsta */
   printf("\n  Previous value was %d.\n", In(adr0));
   Out(adr, v);                                       /* set the new address */
   stat1 = In(isr1);                                  /* update isr1 copy */
   stat2 = In(isr2);                                  /* update isr2 copy */
   return updated(ibsta);
   }

/*
 * IBTMO
 * change the timeout value.  The number given is the number of 
 * times (hex) that ibrd and ibwrt check for the operation to be finished
 * before timing out.  The actual amount of time this takes will
 * depend on the speed of your system.
 */
ibtmo(v)
   unsigned long int v;

   {
   ibcnt = 0;                                               /* no count for this function */
   ibsta = 0;                                               /* clear ibsta */
   printf("\n  Previous timeout value: %lx,  ", timeout);
   timeout = v;
   printf("New value: %lx\n", timeout);
   stat1 = In(isr1);                            /* update isr1 copy */
   stat2 = In(isr2);                            /* update isr2 copy */
   return updated(ibsta);
   }
/*****PUEjP This routine has been added to map interface to VME module on 
 SPARC under SunOS4
*/
int esp488_open(void)
{
  int base,fd,page_size;
  caddr_t page_ptr,ptr;
  unsigned int base_offset, base_page,pagesize,window_size;

  base=ESP488_BASE;

  /* Open the unit */
  fd = open("/dev/vme16d16",O_RDWR);
  if(fd<=0) {
    fprintf(stderr,
	"Error (esp488_open) - failed to open device /dev/vme16d16.\n");
    return(-2);}
  /* memory map the device */
  /* must get the allocated page to line up on a page boundary */
  pagesize = getpagesize();
  base_offset = base%pagesize;
  base_page = base-base_offset;
  page_size = window_size = 
    ((base_offset+ESP488_WINDOW_SIZE)/pagesize+ 1)*pagesize;

  /* map this page to memory */
  page_ptr = mmap(0, window_size,PROT_WRITE | PROT_READ, 
		     MAP_SHARED, fd, base_page); 
  /* did it work? */
  if(page_ptr== (caddr_t) -1) {
    fprintf(stderr,"Error (esp488_open) - failed to map memory.\n");
    return(-3); }
  /* correct back to the actual base of the module */
  ptr = (unsigned char *) page_ptr+base_offset;

  ib = (struct ibregs *) ptr;

  /*  printf("GPIB Interface successfully opened.  IB = 0x%X\n",ib); */

  return(0);
}
/**** ibinit() - initialization routine to call a couple of other routines */
int ibinit(void)
{
  ibonl(1);  /* initialize GPIB */
  ibsic();   /* clear interface */
  ibsre(1);  /* set remote enable */
  if(ibsta&(0x8000)) {
    fprintf(stderr,"Error initializing GPIB. IBSTAT=0x%4.4X\n",ibsta);
  }
  return(updated(ibsta));
}
/* ibsetwr() - set direction to write from add_1014 (usually 0) to
   add_slave.
*/
int ibsetwr(int add_slave)
{
  byte cmdbuf[4];
  int add_1014;

  add_1014 = In(adr0);
  add_1014 &= 0x1F;

  cmdbuf[0] = 0x11;                    /* (LLO) local lock-out */
  cmdbuf[1] = 0x40 | add_1014;         /* interface talk address */
  cmdbuf[2] = 0x14;                    /* (DCL) device clear */
  cmdbuf[3] = 0x20 | add_slave;        /* slave GPIB address */
  return(ibcmd(cmdbuf,4));             /* write to the command buffer */
}
/* ibsetrd() - set direction to read from add_slave to add_1014 (usually 0)
 */
int ibsetrd(int add_slave)
{
  byte cmdbuf[2];
  int add_1014;
  
  add_1014 = In(adr0);
  add_1014 &= 0x1F;

  cmdbuf[0] = 0x20 | add_1014;         /* set 1014 to listen */
  cmdbuf[1] = 0x40 | add_slave;        /* set slave to talk */
  return(ibcmd(cmdbuf,2));             /* write to command buffer */
}
/* ibget() - set a GET command (needed before a read) */
int ibget(void)
{
  byte cmdbyte;
  cmdbyte = 0x08;
  return(ibcmd(&cmdbyte,1));
}
/*******PUEjP End of addition */




















