/*
 * Engineering Software Package GPIB-1014P functions
 * Rev B
 * (c) Copyright 1990, National Instruments
 * All rights reserved.
 *
 * The GPIB functions implemented here are a straightforward
 * set of independent functions that can be used to directly
 * manipulate an interface board.  They are at the level of
 * example code provided in the hardware reference manual but
 * structured to be a subset of the complete integrated driver
 * package.  Sufficient functionality is implemented so that
 * an application program can control and coordinate data
 * transfers among a suite of instruments on the GPIB.  The
 * primary purpose of these functions is to demonstrate the
 * use of the GPIB interface board in an ATE environment so
 * that an engineer or programmer is able to customize the
 * GPIB functions for a dedicated stand-alone or ROM-based
 * system, rather than implement the software from scratch.
 * Whenever an operating system environment is available the
 * recommended approach is to use a specific binary driver or
 * handler package containing the full GPIB capability rather
 * than use this engineering software package.
 *
 * The function calls support synchronous, interrupting,
 * non-DMA I/O, for a single interface board which is always
 * the system controller and the controller in charge.  All
 * functions return the subset of the standard GPIB status
 * bit vector consisting of END,SRQI,CIC,ATN,LACS and TACS status bits.
 * The result of the last call is also available in the global
 * variable, ibsta.  If it was an I/O operation the actual
 * count transferred is available in the global variable,
 * ibcnt.  Before any other call is made the ibonl function
 * must be called to initialize the GPIB interface.  Prior to
 * calling ibrd or ibwrt the appropriate devices, including
 * the interface board, must be addressed by calling ibcmd
 * with the proper addressing commands.
 */


#define TRUE 0x01;
#define FALSE 0x00;

#define IB      ((struct ibregs *) 0x1000)      /* short I/O address of */
						                              /* interface board      */

unsigned ibsta;                 /* status bits returned by last call */
unsigned ibcnt;                 /* actual byte count of last I/O transfer */

unsigned long int timeout = 0xFFFFF;   /* number of loops to count before timing out */

char listening;            /* true if a GPIB read in progress */

char stat1, stat2;         /* software copies of isr1 and isr2; These bits */
                           /* are cleared when read, so each function must */
                           /* update a software copy of them for use in */
                           /* updating ibsta.  If the updated() function */
                           /* actually read these registers, it would */
                           /* clear some bits which might be needed by the */
                           /* next function to be executed. */

int handler();             /* the interrupt handler routine */

char  *str,                /* global pointer for interrupt handler */
      doneflag;            /* set by handler when I/O complete */

int count;                 /* global counter for interrupt handler */


/* Hardware register definitions */

struct ibregs {            /*  write-only hardware registers       */
	char    x0,     cdor;   /*  +1 byte out register                */
	char    x2,     imr1;   /*  +3 interrupt mask register 1        */
	char    x4,     imr2;   /*  +5 interrupt mask register 2        */
	char    x6,     spmr;   /*  +7 serial poll mode register        */
	char    x8,     admr;   /*  +9 address mode register            */
	char    xA,     auxmr;  /*  +B auxiliary mode register          */
	char    xC,     adr;    /*  +D address register 0/1             */
	char    xE,     eosr;   /*  +F end of string register           */
};

/* Read-only register mnemonics corresponding to write-only registers */

#define dir     cdor
#define isr1    imr1
#define isr2    imr2
#define spsr    spmr
#define adsr    admr
#define cptr    auxmr
#define adr0    adr
#define adr1    eosr

/* Control masks for hidden registers (auxmr) */

#define ICR     0040
#define PPR     0140
#define AUXRA   0200
#define AUXRB   0240
#define AUXRE   0300

/* Hardware register bit definitions */
/*      Name            Bit(s)          register       */

#define HR_DI           (1<<0)          /* isr1         */
#define HR_DO           (1<<1)          /* isr1         */
#define HR_END          (1<<4)          /* isr1         */
#define HR_CO           (1<<3)          /* isr2         */
#define HR_SRQI         (1<<6)          /* isr2         */
#define HR_DMAI         (1<<4)          /* imr2         */
#define HR_DMAO         (1<<5)          /* imr2         */
#define HR_ADM0         (1<<0)          /* admr         */
#define HR_TRM0         (1<<4)          /* admr         */
#define HR_TRM1         (1<<5)          /* admr         */
#define HR_DL           (1<<5)          /* adr          */
#define HR_DT           (1<<6)          /* adr          */
#define HR_ARS          (1<<7)          /* adr          */
#define HR_PPU          (1<<4)          /* ppr          */

/* 7210 Auxiliary Commands */

#define AUX_PON         000     /* Immediate Execute pon        */
#define AUX_CR          002     /* Chip Reset                   */
#define AUX_SEOI        006     /* Send EOI                     */
#define AUX_TCA         021     /* Take Control Asynchronously  */
#define AUX_GTS         020     /* Go To Standby                */
#define AUX_EPP         035     /* Execute Parallel Poll        */
#define AUX_SIFC        036     /* Set IFC                      */
#define AUX_CIFC        026     /* Clear IFC                    */
#define AUX_SREN        037     /* Set REN                      */
#define AUX_CREN        027     /* Clear REN                    */


/* I/O macros; The In and Out functions have been defined as these two */
/* macros which pass the I/O address and data width to two functions */
/* called input and output.  This allows you to define your own method */
/* of accessing the registers, since some special steps may need to be */
/* taken if the GPIB-1014 is in another chassis linked by an extender */
/* or some other unusual setup. */

#define In(x)           input((long)(&(IB->x)), 1)
#define Out(x,a)        output((long)(&(IB->x)), (long)a, 1)


input(address, width)               /* reads a register on the GPIB-1014; */
   long address;                    /* This input function is for a cpu */
   int width;                       /* with the same addressing and byte */
                                    /* ordering scheme as the VMEbus. If */
   {                                /* you have an Intel processor or a */
   switch(width)                    /* processor which is incapable of */
      {                             /* 32-bit accesses, you may need to */
      case 1:                       /* write your own input routine. */
         return *(char *)address;
      case 2:
         return *(int *)address;
      case 4:
         return *(long *)address;
      }
   }


output(address, value, width)       /* writes a register on the GPIB-1014 */
   long address, value;             /* This output function is for a cpu */
   int width;                       /* with the same addressing and byte */
                                    /* ordering scheme as the VMEbus. If */
   {                                /* you have an Intel processor or a */
   switch(width)                    /* processor which is incapable of */
      {                             /* 32-bit accesses, you may need to */
      case 1:                       /* write your own output routine. */
         *(char *)address = value;
         break;
      case 2:
         *(int *)address = value;
         break;
      case 4:
         *(long *)address = value;
         break;
      }
   }


unsigned int updated(status)     /* updates the global variable ibsta */
   unsigned int status;

/* the meanings of the bits in ibsta are:
   
   Bit   Mnemonic    Condition flagged

   13    ERR         The operation was unable to complete successfully
   12    ---
   11    END         A read operation was halted by EOI
   10    SRQI        SRQ was asserted at least once since the last operation
   9     ---
   8     ---
   7     ---
   6     ---
   5     CIC         The GPIB-1014 is controller in charge
   4     ATN         ATN is asserted
   3     TACS        The GPIB-1014 is addressed to talk
   2     LACS        The GPIB-1014 is addressed to listen
   1     ---
   0     ---

*/

   {
   ibsta &= 0x8000;                             /* clear all but ERR bit */
   ibsta |= ( (stat1 & 0x04) ? 0x8000:0x00 );   /* ERR */
   ibsta |= ( (In(adsr) & 0x02) ? 0x08:0x00 );  /* TA */
   ibsta |= ( (In(adsr) & 0x04) ? 0x04:0x00 );  /* LA */
   ibsta |= ( (In(adsr) & 0x40) ? 0x00:0x10 );  /* ATN */
   ibsta |= ( (In(adsr) & 0x80) ? 0x20:0x00 );  /* CIC */
   ibsta |= ( (stat2 & 0x40) ? 0x1000:0x00 );   /* SRQI */
   ibsta |= ( (stat1 & 0x10) ? 0x2000:0x00 );   /* END */
   return ibsta;
   }


intenable()          /* enables interrupts */
   {                 /* This function does all of the steps */
   }                 /* required to set up the system to respond */
                     /* to the GPIB-1014P's interrupt.  This is a */
                     /* system-specific function and you will */
                     /* have to write your own intenable() function */
                     /* doing whatever is necessary to set up and */
                     /* enable your interrupt handler. */


intdisable()         /* disables interrupts */
   {                 /* This is a system-specific routine which */
   }                 /* disables the system's response to interrupts. */
                     /* You will have to write your own intdisable() */
                     /* function which undoes the intenable function. */


handler(int level, long status)  /* the interrupt service routine */
   {
   ibcnt++;                                                 /* increment counter */
   if ( ((stat1 = In(isr1)) & HR_END) || (ibcnt == count))
      {
      doneflag = TRUE;        /* notify main program that transfer is complete */
      Out(imr1, 0x00);        /* prevent any further interrupts */
      }
   if (listening)
      str[ibcnt-1] = In(dir);    /* read the byte into buffer */
   else
      Out(cdor, str[ibcnt-1]);   /* send the byte */
   }


/*
 * IBRD
 * Read up to cnt bytes of data from the GPIB into buf.  In
 * addition to I/O complete, the read operation terminates
 * on detection of EOI.  Prior to beginning the read the
 * interface is placed in the controller standby state.  No
 * handshake holdoffs are used so care must be exercised
 * when taking control (i.e., asserting ATN) following ibrd.
 * Prior to calling ibrd, the intended devices as well as
 * the interface board itself must be addressed by calling
 * ibcmd.
 */

ibrd(buf,cnt) char buf[];{
   unsigned long int i;               /* time counter */

	Out(auxmr, AUX_GTS);               /* if active controller, go to standby */
	ibcnt = 0;                         /* reset count */
   ibsta = 0;                         /* clear ibsta */
   listening = TRUE;                  /* we're reading, not writing */
   str = buf;                         /* set pointer for interrupt handler */
   count = cnt;                       /* set handler's count */
	stat1 = 0;                         /* clear our isr1 copy */
   i = timeout;                       /* set timer */
   printf("\n  Waiting for %d bytes...\n", cnt);
   intenable();                     /* prepare the system to handle the interrupts */
   Out(imr1, HR_DI);                /* enable Data In interrupt */

   while (--i && !doneflag);        /* wait for I/O to complete */

   intdisable();                    /* disable response to interrupts */
   if (!i)
      {
      printf("\n  Error - timed out.\n");
      ibsta |= 0x8000;
      }
   stat2 = In(isr2);                /* update isr2 copy */
	return updated(ibsta); }

/*
 * IBWRT
 * Write cnt bytes of data from buf to the GPIB.  The write
 * operation terminates only on I/O complete.  By default,
 * EOI is always sent along with the last byte.  Prior to
 * beginning the write the interface is placed in the
 * controller standby state.  Prior to calling ibwrt, the
 * intended devices as well as the interface board itself
 * must be addressed by calling ibcmd.
 */
ibwrt(buf,cnt) char buf[];{
   unsigned long int i;               /* timeout counter */

	Out(auxmr, AUX_GTS);               /* if CAC, go to standby */
	ibcnt= 0;                          /* reset count */
   ibsta = 0;                         /* clear ibsta */
   stat1 = 0;                         /* clear the isr1 copy */
   i = timeout;                       /* set timer */ 
   listening = FALSE;                 /* let handler know we're talking */
   str = buf;                         /* give interrupt handler pointer to buffer */
   count = cnt;                       /* tell handler the count */
   intenable();                       /* enable response to interrupts */
   Out(imr1, HR_DO);                  /* enable Data Out interrupt */

	while(--i && !doneflag);           /* wait for I/O to complete */

   intdisable();                      /* disable response to interrupts */
   if (!i)
      {
      printf("\n  Error - timed out.\n");
      ibsta |= 0x8000;
      }
   stat2 = In(isr2);                   /* update isr2 copy */
	return updated(ibsta); }

/*
 * IBCMD
 * Write cnt command bytes from buf to the GPIB.  The
 * command operation terminates only on I/O complete.  Prior
 * to beginning the command the interface is placed in the
 * controller active state.  Before calling ibcmd for the
 * first time, ibsic must be called to initialize the GPIB
 * and enable the interface to leave the controller idle
 * state.
 */
ibcmd(buf,cnt) char buf[];{

	Out(auxmr, AUX_TCA);                       /* if standby, go to CAC */
	ibcnt = 0;                                 /* clear ibcnt */
   ibsta = 0;                                 /* clear ibsta */
	stat2 = 0;
	while(ibcnt<cnt){
		stat2 &= ~HR_CO;                          /* clear saved copy of Command Out */
		Out(cdor, buf[ibcnt++]);                  /* output command and increment count */
		while(((stat2 |= In(isr2)) & HR_CO)==0) ; /* wait for Command Out; "or" to preserve SRQI */
		}
   stat1 = In(isr1);                            /* update isr1 copy */
	return updated(ibsta); }

/*
 * IBWAIT
 * Check or wait for a GPIB event to occur.  The mask argument
 * is a bit vector corresponding to the status bit vector.  It
 * has a bit set for each condition which can terminate the wait
 * (not all bits in this implementation).  If the mask is 0 then no
 * condition is waited for and the current status is simply
 * returned.  Note that since the hardware SRQI bit is volatile
 * it will only be reported once for each occurrence of SRQ.  If
 * another function has returned the SRQI status bit, then a
 * call to ibwait with a mask containing SRQI will never return.
 */
ibwait(mask){
   ibcnt = 0;                                /* no count for this function */
   ibsta = 0;                                /* clear ibsta */
   do
      {
      stat1 = In(isr1);                      /* update isr1 copy */
      stat2 = In(isr2);                      /* update isr2 copy */
      }
	while(!(updated(ibsta) & mask) && mask);  /* quit if pattern matched or mask = 0 */
	return ibsta; }


/*
 * IBRPP
 * Conduct a parallel poll and return the byte in buf.  Prior
 * to conducting the poll the interface is placed in the
 * controller active state.
 */
ibrpp(buf) char *buf;{
   ibcnt = 0;                             /* no count for this function */
   ibsta = 0;                             /* clear ibsta */
	Out(auxmr, AUX_TCA);                   /* if standby, go to CAC */
	Out(auxmr, AUX_EPP);                   /* enable parallel poll */
   while (!((stat2 = In(isr2)) & HR_CO)); /* wait for the CO bit to be set */
	*buf= In(cptr);                        /* store the response byte */
   stat1 = In(isr1);                      /* update isr1 copy */
	return updated(ibsta); }

/*
 * IBONL
 * Initialize the interface hardware.  If v is non-zero then
 * the GPIB chip is enabled online.  If v is zero then it is
 * left disabled and offline.  Ibonl must be called before
 * any other function.
 */
ibonl(v){

   ibcnt = 0;              /* no count for this function */
   ibsta = 0;              /* clear ibsta */
	Out(auxmr, AUX_CR);     /* reset chip */
	stat1 = In(isr1) && 0;  /* clear status registers by reading */
	stat2 = In(isr2) && 0;
	Out(imr1, 0);           /* disable all interrupts */
	Out(imr2, 0);
	Out(spmr, 0);                           /* reset serial poll response */
	Out(adr, 0);                            /* set GPIB address; MTA= 00, MLA= 00 */
	Out(adr, HR_ARS | HR_DT | HR_DL);       /* disable secondary addressing */
	Out(admr, HR_TRM1 | HR_TRM0 | HR_ADM0); /* enable dual primary addressing mode */
	Out(eosr, 0);
	Out(auxmr, AUX_CIFC);           /* by default enable system controller */
	Out(auxmr, ICR | 8);            /* set internal counter register N= 8 */
	Out(auxmr, PPR | HR_PPU);       /* parallel poll unconfigure */
	Out(auxmr, AUXRA | 0);          /* clear auxmr hidden registers */
	Out(auxmr, AUXRB | 0);
	Out(auxmr, AUXRE | 0);
	if(v) Out(auxmr, AUX_PON);      /* release pon state to bring online */
	return updated(ibsta); }


/*
 * IBSIC
 * Send IFC for at least 100 microseconds.  Ibsic must be
 * called prior to the first call to ibcmd in order to
 * initialize the bus and enable the interface to leave
 * the controller idle state.
 */
ibsic(){
	int i;

   ibcnt = 0;                 /* no count for this function */
   ibsta = 0;                 /* clear ibsta */
	Out(auxmr, AUX_SIFC);      /* assert IFC */
	for(i= 0; i<200; i++) ;    /* busy wait >=100us */
	Out(auxmr, AUX_CIFC);      /* clear IFC */
   stat1 = In(isr1);          /* update isr1 copy */
   stat2 = In(isr2);          /* update isr2 copy */
	return updated(ibsta); }

/*
 * IBSRE
 * Send REN true if v is non-zero or false if v is zero.
 */
ibsre(v){
   ibcnt = 0;                              /* no count for this function */
   ibsta = 0;                              /* clear ibsta */
	Out(auxmr, (v? AUX_SREN : AUX_CREN));   /* set or clear REN */
   stat1 = In(isr1);                       /* update isr1 copy */
   stat2 = In(isr2);                       /* update isr2 copy */
	return updated(ibsta); }

/*
 * IBGTS
 * Go to the controller standby state from the controller
 * active state, i.e., turn ATN off.
 */
ibgts(){
   ibcnt = 0;                 /* no count for this function */
   ibsta = 0;                 /* clear ibsta */
	Out(auxmr, AUX_GTS);       /* go to standby */
   stat1 = In(isr1);          /* update isr1 copy */
   stat2 = In(isr2);          /* update isr2 copy */
	return updated(ibsta); }

/*
 * IBCAC
 * Return to the controller active state from the
 * controller standby state, i.e., turn ATN on.  Note
 * that in order to enter the controller active state
 * from the controller idle state, ibsic must be called.
 */
ibcac(){
   ibcnt = 0;                 /* no count for this function */
   ibsta = 0;                 /* clear ibsta */
	Out(auxmr, AUX_TCA);       /* assert ATN (asynchronously) */
   stat1 = In(isr1);          /* update isr1 copy */
   stat2 = In(isr2);          /* update isr2 copy */
	return updated(ibsta); }


/*
 * IBPAD
 * change the GPIB address of the interface board.
 * The address must be 0 through 31. ibonl resets the address to 0.
 */
ibpad(v)
   int v;

   {
   ibcnt = 0;                                         /* no count for this function */
   ibsta = 0;                                         /* clear ibsta */
   printf("\n  Previous value was %d.\n", In(adr0));
   Out(adr, v);                                       /* set the new address */
   stat1 = In(isr1);                                  /* update isr1 copy */
   stat2 = In(isr2);                                  /* update isr2 copy */
   return updated(ibsta);
   }

/*
 * IBTMO
 * change the timeout value.  The number given is the number of 
 * times that ibrd and ibwrt check for the operation to be finished
 * before timing out.  The actual amount of time this takes will
 * depend on the speed of your system.
 */
ibtmo(v)
   unsigned long int v;

   {
   ibcnt = 0;                                               /* no count for this function */
   ibsta = 0;                                               /* clear ibsta */
   printf("\n  Previous timeout value: %lx,  ", timeout);
   timeout = v;
   printf("New value: %lx\n", timeout);
   stat1 = In(isr1);                            /* update isr1 copy */
   stat2 = In(isr2);                            /* update isr2 copy */
   return updated(ibsta);
   }
