/* -------------------------------------------------------------------------*/
/* program to use the GPIB-1014 ESP calls from a screen prompt. */


/* --------------------------- include files ------------------------------ */


/* ------------------------------ constants --------------------------------*/

#define MAXLEN  0x1000        /* buffer size (largest possible transfer) */
#define STRLEN 0x80           /* length of string input at prompt */

#define TRUE 1
#define FALSE 0


/* --------------------- function declarations -----------------------------*/

int doibcmd(), doibonl(), doibsic(), doibgts(), doibcac(),
    doibrd(), doibwrt(), doibsre(), doibwait(), doibrpp(),
    doibtmo(), doibpad(), dohelp();


/* --------------------------- global variables ----------------------------*/

extern unsigned int  ibsta,      /* the GPIB transfer status byte */
                     ibcnt;      /* the number of bytes transferred */

char  buf[MAXLEN];   /* buffer for reading and writing */

char line[STRLEN];      /* line entered at mxibic prompt */
char previous[STRLEN];  /* previous line */

char *pos;     /* current position within line */

/* call structure contains the name of a GPIB function and the address */
/* of the c function called to access it */

struct call {
            char name[7];
            int (*addr)();
            };

/* calls is the array of all the function descriptors */
            
struct call calls[] = {
                      { "ibcmd", doibcmd },
                      { "ibonl", doibonl },
                      { "ibsic", doibsic },
                      { "ibgts", doibgts },
                      { "ibcac", doibcac },
                      { "ibrd", doibrd },
                      { "ibwrt", doibwrt },
                      { "ibsre", doibsre },
                      { "ibrpp", doibrpp },
                      { "ibwait", doibwait },
                      { "ibtmo", doibtmo },
                      { "ibpad", doibpad },
                      { "help", dohelp },
                      { 0, 0 },
                      };


/* ------------------ main program and subroutines -------------------------*/

main()
   {
   printtitle();              /* print the title lines */
   followorders();            /* get commands and execute them */
   ibonl(0);                  /* take the GPIB-1014 offline */
   }
   

printtitle()
   {
   printf("\n\n");
   printf("                    National Instruments\n");
   printf("IEEE-488 Miniature Interface Bus Interactive Control program (minibic)\n");
   printf("      Copyright (C) 1990 National Instruments, Inc.  Version 1.0\n");
   printf("\n");
   printf("                    Type 'help' for help.\n");
   printf("          Use lowercase letters only for function names.\n");
   }


followorders()          /* execute commands from prompt */
   {
   char quitting = FALSE;  /* true if quit command received */
   char found,             /* true if valid function typed */
        count;             /* position within calls array */
   int  i,                 /* index */
        len;               /* length of current function name */
   char name[8];           /* current function name */

   do
      {
      printf("\n: ");
      gets(line);
      if ( *line == 'q' )
         quitting = TRUE;
      if ( *line == '!' )
         strcpy(line, previous);
      count = 0;
      found = FALSE;
      while ( len = strlen(calls[count].name) )   /* search until 0 is reached */
         {
         pos = line;
         while ( *pos == ' ' || *pos == '\t' )     /* skip leading whitespace */
            pos++;
         for (i=0; i<len; i++)  /* make name contain the function name typed from the prompt */
            {
            name[i] = *pos;
            pos++;
            }
         name[len] = '\0';
         if ( !strcmp(name, calls[count].name) )
            {
            found = TRUE;
            strcpy(previous, line);
            (*calls[count].addr)();
            printf("\n  ibsta = %x\t\tibcnt = %d\n", ibsta, ibcnt);
            }
         count++;
         }
      if (!quitting && !found)
         printf("\nSyntax error!\n");
      }
      while (!quitting);
   }


doibcmd()
   {
   removequotes();
   ibcmd(pos, strlen(pos));
   }


doibonl()
   {
   int v;         /* zero or not; off or on */

   sscanf(pos, "%d", &v);
   ibonl(v);
   }


doibsic()
   {
   ibsic();
   }


doibgts()
   {
   ibgts();
   }


doibcac()
   {
   ibcac();
   }


doibwrt()
   {
   removequotes();
   ibwrt(pos, strlen(pos));
   }


doibrd()
   {
   char string[STRLEN];
   int v;

   sscanf(pos, "%d", &v);
   ibrd(string, v);
   string[ibcnt] = '\0';
   printf("\n   %s\n", string);
   }


doibrpp()
   {
   unsigned char buf;

   printf("\nibrpp\n");
   ibrpp(&buf);
   printf("\n   %02x\n", buf);
   }


doibsre()
   {
   int v;

   sscanf(pos, "%d", &v);
   ibsre(v);
   }


doibwait()
   {
   unsigned int mask;

   sscanf(pos, "%x", &mask);
   printf("\n  Waiting for ibsta bits set %x...\n", mask);
   ibwait(mask);
   printf("  Done.\n");
   }


doibtmo()
   {
   unsigned long int v;

   sscanf(pos, "%lx", &v);
   ibtmo(v);
   }


doibpad()
   {
   int v;

   sscanf (pos, "%x", &v);
   ibpad(v);
   }


dohelp()
   {
   printf("\n");
   printf("  ibonl v         v=0 disables board, v nonzero initializes it.\n");
   printf("  ibsic           sends IFC and takes control.\n");
   printf("  ibcmd \"string\"  sends ASCII bytes in string as commands.\n");
   printf("  ibwrt \"string\"  writes the ASCII string to the GPIB.\n");
   printf("  ibrd cnt        reads cnt bytes from the GPIB.\n");
   printf("  ibcac           asserts ATN and becomes active controller.\n");
   printf("  ibgts           releases ATN and goes to standby.\n");
   printf("  ibsre v         v=0 clears REN, v nonzero asserts REN.\n");
   printf("  ibpad v         sets GPIB address to v, 0 <= v <= 31.\n");
   printf("  ibtmo v         sets timeout to v, v is a large hex number.\n");
   printf("  ibrpp           conducts a parallel poll and prints the hex response.\n");
   printf("  ibwait v        waits for ibsta to have the same bits set\n");
   printf("                  as v, a 4-digit hex number.\n");
   printf("  !               repeats the last line entered.\n");
   printf("  q               quit minibic.\n");
   }


removequotes()
   {
   char *new;

   while ( *pos != '"' )
      pos++;
   new = ++pos;
   while ( *new != '"' )
      new++;
   *new = '\0';
   }
