/*  Set of routines to use the HP 34401A DVM via the GPIB interfact */                      
/* using the national GPIB driver and libraries                     */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

extern unsigned ibsta, ibcnt;

/* some definitions. */

#define HPDVM_GPIB 0                 /* gpib driver number */
#define HPDVM_GPIB_PRIMADD 22        /* HP DVM primary GPIB address */

#define UNL  0x3f  /* GPIB unlisten command                */
#define UNT  0x5f  /* GPIB untalk command                  */
#define GTL  0x01  /* GPIB go to local                     */
#define SDC  0x04  /* GPIB selected device clear           */
#define PPC  0x05  /* GPIB parallel poll configure         */
#define GET  0x08  /* GPIB group execute trigger           */
#define TCT  0x09  /* GPIB take control                    */
#define LLO  0x11  /* GPIB local lock out                  */
#define DCL  0x14  /* GPIB device clear                    */
#define PPU  0x15  /* GPIB parallel poll unconfigure       */
#define SPE  0x18  /* GPIB serial poll enable              */
#define SPD  0x19  /* GPIB serial poll disable             */
#define PPE  0x60  /* GPIB parallel poll enable            */
#define PPD  0x70  /* GPIB parallel poll disable           */

/* GPIB status bit vector :                                */
/*       global variable ibsta and wait mask               */

#define ERR     (1<<15) /* Error detected                  */
#define TIMO    (1<<14) /* Timeout                         */
#define END     (1<<13) /* EOI or EOS detected             */
#define SRQI    (1<<12) /* SRQ detected by CIC             */
#define RQS     (1<<11) /* Device needs service            */
#define SPOLL   (1<<10) /* Board has been serially polled  */
#define EVENT   (1<<9)  /* An event has occured            */
#define CMPL    (1<<8)  /* I/O completed                   */
#define LOK     (1<<7)  /* Local lockout state             */
#define REM     (1<<6)  /* Remote state                    */
#define CIC     (1<<5)  /* Controller-in-Charge            */
#define ATN     (1<<4)  /* Attention asserted              */
#define TACS    (1<<3)  /* Talker active                   */
#define LACS    (1<<2)  /* Listener active                 */
#define DTAS    (1<<1)  /* Device trigger state            */
#define DCAS    (1<<0)  /* Device clear state              */

/* some global variables */


unsigned char hpdvm_rd[512],hpdvm_wr[512];    /* read and write buffers */
int esp488_talk_address;
int esp488_listen_address;
int hpdvm_talk_address;
int hpdvm_listen_address;
unsigned char cmdbuf[256];

void gpiberr(char *);

int hpdvm_init_voltread(float range, float res, int trig_mode)
/* Routine to setup the dvm to read voltages. Arguments:
*      range - full scale range (in volts)
*      res   - resolution (in volts). 0 -> set to MIN.
*      trig_mode - 0-> set for immediate triggers
*                  1-> set for external triggers
*
*   returns:
*      0-> all OK
*     -1-> GPIB error
*     -2-> HP DVM error (not yet implemented)
*/
{

/* get a handle for the DVM.  If fails here, probably not powered    
   up or disconnected. */

  /*    hpdvm_handle = ibdev (HPDVM_GPIB, 
                          HPDVM_GPIB_PRIMADD,
                          HPDVM_GPIB_SECADD, 
                          HPDVM_TIMEOUT_CODE, 1, 0);

    if (hpdvm_handle < 0) 
      {
      gpiberr("ibdev Error");
      return(-1);
      }
      */

  if(ibinit()&ERR) {
    fprintf(stderr,"Error initializing gpib interface.\n");
    exit(-1);
  }
  /* ibonl(1);  *//* initialize HPIB */
  /* ibsic(); */  /* send IFC to clear the GPIB */
  /*  ibsre(1);  *//* set the Remote enable bit */
    if (ibsta & ERR) {
      gpiberr("Initilization Error");
      return(-1);
      }

    /* Now set the talk and listen addresses */
    esp488_talk_address = 0x40 | HPDVM_GPIB;
    esp488_listen_address = 0x20 | HPDVM_GPIB;
    hpdvm_talk_address = 0x40 | HPDVM_GPIB_PRIMADD;
    hpdvm_listen_address = 0x20 | HPDVM_GPIB_PRIMADD;
    /* Now the initialization sequence */
    cmdbuf[0] = LLO;
    cmdbuf[1] = esp488_talk_address;
    cmdbuf[2] = DCL;
    cmdbuf[3] = hpdvm_listen_address;
    /* Send this to the command buffer */
    /*ibcmd(cmdbuf,4); */
    ibsetwr(HPDVM_GPIB_PRIMADD);
     printf("Initialization complete IBSTA = 0x%X\n",ibsta);
    
    ibwrt ("*RST",4L); 
    if (ibsta & ERR) {
      gpiberr("*RST Error");
      return(-1);
      }

    ibwrt ("*CLS",4L); 
    if (ibsta & ERR) {
      gpiberr("*CLR Error");
      return(-1);
      }

    if(res>0.)
      {
      sprintf(hpdvm_wr,"CONF:VOLT:DC %f, %f",range,res);
      }
    else
      {
      sprintf(hpdvm_wr,"CONF:VOLT:DC %f, MIN",range);
      }
    ibwrt (hpdvm_wr, (long) strlen(hpdvm_wr));
    
    if (ibsta & ERR) {
      gpiberr("CONF Error");
      return(-1);
      }

    if(trig_mode==0)
      {
      ibwrt ("TRIG:SOUR IMM",13L);
      }
    else  
      {
      ibwrt ("TRIG:SOUR EXT",13L);
      }

    if (ibsta & ERR) {
      gpiberr("TRIG Error");
      return(-1);
      }
/* If here, then everything probably OK */
  return(0);
}
int hpdvm_read(double *value) 
/* Routine to read a value from the HP dvm.  What the value is
*   depends on how the meter was configured.
*
*      Arguments:
*        value - address of the return value
*
*      Returns:
*        0 -> everything OK
*        1 -> device timed out (external trig, no trig for 30 seconds)
*       -1 -> GPIB error
*       -2 -> DVM error (not yet implemented)
*/
{
    unsigned char stb;
    *value = 0.;

    ibsetwr(HPDVM_GPIB_PRIMADD);  /* set direction to write */
    ibwrt ("*CLS",4L);            /* clear any pending errors */
    if(ibsta & ERR) {
      gpiberr("CLS Error");
      return(-1);
      }


    ibwrt ("READ?",5L);
    if(ibsta & ERR) {
      gpiberr("READ? Error");
      return(-1);
    }

    ibget();                     /* issue a GET to fetch result */
    /* switch the direction */
    ibsetrd(HPDVM_GPIB_PRIMADD); 
  

    ibrd (hpdvm_rd,255L);        /* read return buffer */
    if (ibsta & ERR) {
      if(ibsta&TIMO) return(1);   /* handle timeout differently */
      gpiberr("ibrd Error");
      return(-3);
      }

    hpdvm_rd[ibcnt]='\0';        /* properly terminate string */
    *value = atof(hpdvm_rd);

    return(0);
}
/* ===========================================================================
 *                      Function GPIBERR
 *  This function will notify you that a NI-488 function failed by
 *  printing an error message.  The status variable IBSTA will also be
 *  printed in hexadecimal along with the mnemonic meaning of the bit position. 
 *  The status variable IBERR will be printed in decimal along with the
 *  mnemonic meaning of the decimal value.  The status variable IBCNT will
 *  be printed in decimal.
 *
 *
 *  The EXIT function will terminate this program.
 * ===========================================================================
 */

void gpiberr(char *msg) {

    printf ("%s\n", msg);
 
    printf ("ibsta = &H%x  <", ibsta);
    if (ibsta & ERR )  printf (" ERR");
    if (ibsta & TIMO)  printf (" TIMO");
    if (ibsta & END )  printf (" END");
    if (ibsta & SRQI)  printf (" SRQI");
    if (ibsta & RQS )  printf (" RQS");
    if (ibsta & SPOLL) printf (" SPOLL");
    if (ibsta & EVENT) printf (" EVENT");
    if (ibsta & CMPL)  printf (" CMPL");
    if (ibsta & LOK )  printf (" LOK");
    if (ibsta & REM )  printf (" REM");
    if (ibsta & CIC )  printf (" CIC");
    if (ibsta & ATN )  printf (" ATN");
    if (ibsta & TACS)  printf (" TACS");
    if (ibsta & LACS)  printf (" LACS");
    if (ibsta & DTAS)  printf (" DTAS");
    if (ibsta & DCAS)  printf (" DCAS");
    printf (" >\n");             
    /* 
    printf ("iberr = %d", iberr);
    if (iberr == EDVR) printf (" EDVR <DOS Error>\n");
    if (iberr == ECIC) printf (" ECIC <Not CIC>\n");
    if (iberr == ENOL) printf (" ENOL <No Listener>\n");
    if (iberr == EADR) printf (" EADR <Address error>\n");
    if (iberr == EARG) printf (" EARG <Invalid argument>\n");
    if (iberr == ESAC) printf (" ESAC <Not Sys Ctrlr>\n");
    if (iberr == EABO) printf (" EABO <Op. aborted>\n");
    if (iberr == ENEB) printf (" ENEB <No GPIB board>\n");
    if (iberr == EOIP) printf (" EOIP <Async I/O in prg>\n");
    if (iberr == ECAP) printf (" ECAP <No capability>\n");
    if (iberr == EFSO) printf (" EFSO <File sys. error>\n");
    if (iberr == EBUS) printf (" EBUS <Command error>\n");
    if (iberr == ESTB) printf (" ESTB <Status byte lost>\n");
    if (iberr == ESRQ) printf (" ESRQ <SRQ stuck on>\n");
    if (iberr == ETAB) printf (" ETAB <Table Overflow>\n");
    */

    printf ("ibcnt = %d\n", ibcnt);
    printf ("\n");


    return;

}


