/*  Set of routines to use the HP 34401A DVM via the GPIB interfact */                      
/* using the national GPIB driver and libraries                     */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*  DECL.H contains constants, declarations, and function prototypes for the      
    GPIB library */

#include <decl.h>

/* 
 *  GPIBERR is an error function that is called when a NI-488 function fails.
 */

void gpiberr(char *msg);           

/* some definitions. */

#define HPDVM_GPIB 0                 /* gpib driver number */
#define HPDVM_GPIB_PRIMADD 22        /* HP DVM primary GPIB address */
#define HPDVM_GPIB_SECADD 0          /* HP DVM secondary address */

#define HPDVM_TIMEOUT_CODE T30s      /* set timeout code to 30 seconds */

/* some global variables */

int hpdvm_handle;                    /* handle assigned to DVM */

char hpdvm_rd[512],hpdvm_wr[512];    /* read and write buffers */

int hpdvm_init_voltread(float range, float res, int trig_mode)
/* Routine to setup the dvm to read voltages. Arguments:
*      range - full scale range (in volts)
*      res   - resolution (in volts). 0 -> set to MIN.
*      trig_mode - 0-> set for immediate triggers
*                  1-> set for external triggers
*
*   returns:
*      0-> all OK
*     -1-> GPIB error
*     -2-> HP DVM error (not yet implemented)
*/
{

/* get a handle for the DVM.  If fails here, probably not powered    
   up or disconnected. */

    hpdvm_handle = ibdev (HPDVM_GPIB, 
                          HPDVM_GPIB_PRIMADD,
                          HPDVM_GPIB_SECADD, 
                          HPDVM_TIMEOUT_CODE, 1, 0);

    if (hpdvm_handle < 0) 
      {
      gpiberr("ibdev Error");
      return(-1);
      }

    ibclr (hpdvm_handle);
    if (ibsta & ERR) {
      gpiberr("ibclr Error");
      return(-1);
      }
/*
 */

    ibwrt (hpdvm_handle,"*RST",4L); 
    if (ibsta & ERR) {
      gpiberr("*RST Error");
      return(-1);
      }

    ibwrt (hpdvm_handle,"*CLS",4L); 
    if (ibsta & ERR) {
      gpiberr("*CLR Error");
      return(-1);
      }

    if(res>0.)
      {
      sprintf(hpdvm_wr,"CONF:VOLT:DC %f, %f",range,res);
      }
    else
      {
      sprintf(hpdvm_wr,"CONF:VOLT:DC %f, MIN",range);
      }
    ibwrt (hpdvm_handle,hpdvm_wr, (long) strlen(hpdvm_wr));
    
    if (ibsta & ERR) {
      gpiberr("CONF Error");
      return(-1);
      }

    if(trig_mode==0)
      {
      ibwrt (hpdvm_handle,"TRIG:SOUR IMM",13L);
      }
    else  
      {
      ibwrt (hpdvm_handle,"TRIG:SOUR EXT",13L);
      }

    if (ibsta & ERR) {
      gpiberr("TRIG Error");
      return(-1);
      }
/* If here, then everything probably OK */
  return(0);
}
int hpdvm_read(double *value) 
/* Routine to read a value from the HP dvm.  What the value is
*   depends on how the meter was configured.
*
*      Arguments:
*        value - address of the return value
*
*      Returns:
*        0 -> everything OK
*        1 -> device timed out (external trig, no trig for 30 seconds)
*       -1 -> GPIB error
*       -2 -> DVM error (not yet implemented)
*/
{
    *value = 0.;

    ibwrt (hpdvm_handle,"*CLS",4L);  /* clear any pending errors */
    if(ibsta & ERR) {
      gpiberr("CLS Error");
      return(-1);
      }


    ibwrt (hpdvm_handle,"READ?",5L);
    if(ibsta & ERR) {
      gpiberr("READ? Error");
      return(-1);
      }

    ibrd (hpdvm_handle,hpdvm_rd,255L);
    if (ibsta & ERR) {
      if(ibsta&TIMO) return(1);   /* handle timeout differently */
      gpiberr("ibrd Error");
      return(-1);
      }

    hpdvm_rd[ibcnt]='\0';
    *value = atof(hpdvm_rd);
    return(0);
}
/* ===========================================================================
 *                      Function GPIBERR
 *  This function will notify you that a NI-488 function failed by
 *  printing an error message.  The status variable IBSTA will also be
 *  printed in hexadecimal along with the mnemonic meaning of the bit position. 
 *  The status variable IBERR will be printed in decimal along with the
 *  mnemonic meaning of the decimal value.  The status variable IBCNT will
 *  be printed in decimal.
 *
 *
 *  The EXIT function will terminate this program.
 * ===========================================================================
 */

void gpiberr(char *msg) {

    printf ("%s\n", msg);
 
    printf ("ibsta = &H%x  <", ibsta);
    if (ibsta & ERR )  printf (" ERR");
    if (ibsta & TIMO)  printf (" TIMO");
    if (ibsta & END )  printf (" END");
    if (ibsta & SRQI)  printf (" SRQI");
    if (ibsta & RQS )  printf (" RQS");
    if (ibsta & SPOLL) printf (" SPOLL");
    if (ibsta & EVENT) printf (" EVENT");
    if (ibsta & CMPL)  printf (" CMPL");
    if (ibsta & LOK )  printf (" LOK");
    if (ibsta & REM )  printf (" REM");
    if (ibsta & CIC )  printf (" CIC");
    if (ibsta & ATN )  printf (" ATN");
    if (ibsta & TACS)  printf (" TACS");
    if (ibsta & LACS)  printf (" LACS");
    if (ibsta & DTAS)  printf (" DTAS");
    if (ibsta & DCAS)  printf (" DCAS");
    printf (" >\n");             
 
    printf ("iberr = %d", iberr);
    if (iberr == EDVR) printf (" EDVR <DOS Error>\n");
    if (iberr == ECIC) printf (" ECIC <Not CIC>\n");
    if (iberr == ENOL) printf (" ENOL <No Listener>\n");
    if (iberr == EADR) printf (" EADR <Address error>\n");
    if (iberr == EARG) printf (" EARG <Invalid argument>\n");
    if (iberr == ESAC) printf (" ESAC <Not Sys Ctrlr>\n");
    if (iberr == EABO) printf (" EABO <Op. aborted>\n");
    if (iberr == ENEB) printf (" ENEB <No GPIB board>\n");
    if (iberr == EOIP) printf (" EOIP <Async I/O in prg>\n");
    if (iberr == ECAP) printf (" ECAP <No capability>\n");
    if (iberr == EFSO) printf (" EFSO <File sys. error>\n");
    if (iberr == EBUS) printf (" EBUS <Command error>\n");
    if (iberr == ESTB) printf (" ESTB <Status byte lost>\n");
    if (iberr == ESRQ) printf (" ESRQ <SRQ stuck on>\n");
    if (iberr == ETAB) printf (" ETAB <Table Overflow>\n");
 
    printf ("ibcnt = %d\n", ibcnt);
    printf ("\n");


    return;

}


